@@ -43,7 +43,7 @@ escapeprint (char *str, int len)
 {
   int i;
 
-  printf ("\t ;; `");
+  printf ("\t;; `");
   for (i = 0; i < len; i++)
     {
       if (((str[i] & 0xFF) >= 'A' && (str[i] & 0xFF) <= 'Z') ||
@@ -62,14 +62,14 @@ hexprint (char *str, int len)
 {
   int i;
 
-  printf ("\t ;; ");
+  printf ("\t;; ");
   for (i = 0; i < len; i++)
     {
       printf ("%02x ", (str[i] & 0xFF));
       if ((i + 1) % 8 == 0)
 	printf (" ");
       if ((i + 1) % 16 == 0 && i + 1 < len)
-	printf ("\n\t ;; ");
+	printf ("\n\t;; ");
     }
 }
 
@@ -78,7 +78,7 @@ binprint (char *str, int len)
 {
   int i;
 
-  printf ("\t ;; ");
+  printf ("\t;; ");
   for (i = 0; i < len; i++)
     {
       printf ("%d%d%d%d%d%d%d%d ",
@@ -92,7 +92,7 @@ binprint (char *str, int len)
       if ((i + 1) % 3 == 0)
 	printf (" ");
       if ((i + 1) % 6 == 0 && i + 1 < len)
-	printf ("\n\t ;; ");
+	printf ("\n\t;; ");
     }
 }
 
@@ -101,16 +101,16 @@ struct stringprep
   char *in;
   int flags;
   char *out;
-  stringprep_profile *profile;
+  Stringprep_profile *profile;
   int rc;
 }
 strprep[] =
 {
-  /* nothing */
+  /* map to nothing U+00AD */
   {
   "foo\xC2\xAD" "bar", 0, "foobar", stringprep_generic}
   ,
-    /* case_nfkc + normalization: */
+    /* map case_nfkc + normalization: */
   {
   "\xC2\xB5", 0, "\xCE\xBC", stringprep_generic}
   ,
@@ -121,21 +121,50 @@ strprep[] =
   {
   "\xC2\xAA", 0, "\x61", stringprep_generic}
   ,
-  /* unassigned code point 0221: */
+  /* unassigned code point U+0221: */
   {
     "\xC8\xA1", 0, "\xC8\xA1", stringprep_generic}
   ,
-  /* unassigned code point 0221: */
+  /* unassigned code point U+0221: */
   {
-    "\xC8\xA1", STRINGPREP_NO_UNASSIGNED, NULL, stringprep_generic, STRINGPREP_CONTAINS_UNASSIGNED}
+    "\xC8\xA1", STRINGPREP_NO_UNASSIGNED, NULL, stringprep_generic,
+    STRINGPREP_CONTAINS_UNASSIGNED}
   ,
-  /* unassigned code point 0236: */
+  /* unassigned code point U+0236: */
   {
     "\xC8\xB6", 0, "\xC8\xB6", stringprep_generic}
   ,
-  /* unassigned code point 0236: */
+  /* unassigned code point U+0236: */
   {
-    "\xC8\xB6", STRINGPREP_NO_UNASSIGNED, NULL, stringprep_generic, STRINGPREP_CONTAINS_UNASSIGNED}
+    "\xC8\xB6", STRINGPREP_NO_UNASSIGNED, NULL, stringprep_generic,
+    STRINGPREP_CONTAINS_UNASSIGNED},
+  /* prohibited ASCII character U+0020: */
+  {
+    "\x20", 0, NULL, stringprep_generic,
+    STRINGPREP_CONTAINS_PROHIBITED},
+  /* prohibited character U+00A0: */
+  {
+    "\xC2\xA0", 0, NULL, stringprep_generic,
+    STRINGPREP_CONTAINS_PROHIBITED},
+  /* prohibited non-character U+10FFFE: */
+  {
+    "\xF4\x8F\xBF\xBE", 0, NULL, stringprep_generic,
+    STRINGPREP_CONTAINS_PROHIBITED},
+  /* prohibited surrogate character U+D801: */
+  {
+    "\xED\xA0\x81", 0, NULL, stringprep_generic,
+    STRINGPREP_CONTAINS_PROHIBITED},
+  /* bidi RandALCat without trailing RandALCat <U+0627><U+0031>: */
+  {
+    "\xD8\xA7\x31", 0, NULL, stringprep_generic,
+    STRINGPREP_BIDI_LEADTRAIL_NOT_RAL},
+  /* bidi RandALCat correct  <U+0627><U+0031><U+0628>: */
+  {
+    "\xD8\xA7\x31\xD8\xA8", 0, "\xD8\xA7\x31\xD8\xA8", stringprep_generic},
+  /* bidi both RandALCat and LCat  <U+0627><U+00AA><U+0628>: */
+  {
+    "\xD8\xA7\xC2\xAA\xD8\xA8", 0, NULL, stringprep_generic,
+    STRINGPREP_BIDI_BOTH_L_AND_RAL}
 };
 
 int
@@ -172,13 +201,6 @@ main (int argc, char *argv[])
 
       strcpy (p, strprep[i].in);
 
-      rc = stringprep (p, BUFSIZ, strprep[i].flags, strprep[i].profile);
-      if (rc != strprep[i].rc)
-	{
-	  fail ("stringprep() entry %d failed: %d\n", i, rc);
-	  continue;
-	}
-
       if (debug)
 	{
 	  printf ("flags: %d\n", strprep[i].flags);
@@ -189,27 +211,36 @@ main (int argc, char *argv[])
 	  puts ("");
 	  binprint (strprep[i].in, strlen (strprep[i].in));
 	  puts ("");
-
-	  if (rc == STRINGPREP_OK)
-	    {
-	      printf ("out:\n");
-	      escapeprint (p, strlen (p));
-	      hexprint (p, strlen (p));
-	      puts ("");
-	      binprint (p, strlen (p));
-	      puts ("");
-
-	      printf ("expected out:\n");
-	      escapeprint (strprep[i].out, strlen (strprep[i].out));
-	      hexprint (strprep[i].out, strlen (strprep[i].out));
-	      puts ("");
-	      binprint (strprep[i].out, strlen (strprep[i].out));
-	      puts ("");
-	    }
-	  else
-	    printf("returned %d expected %d\n", rc, strprep[i].rc);
 	}
 
+      rc = stringprep (p, BUFSIZ, strprep[i].flags, strprep[i].profile);
+      if (rc != strprep[i].rc)
+	{
+	  fail ("stringprep() entry %d failed: %d\n", i, rc);
+	  if (debug)
+	    printf ("FATAL\n");
+	  continue;
+	}
+
+      if (debug && rc == STRINGPREP_OK)
+	{
+	  printf ("out:\n");
+	  escapeprint (p, strlen (p));
+	  hexprint (p, strlen (p));
+	  puts ("");
+	  binprint (p, strlen (p));
+	  puts ("");
+
+	  printf ("expected out:\n");
+	  escapeprint (strprep[i].out, strlen (strprep[i].out));
+	  hexprint (strprep[i].out, strlen (strprep[i].out));
+	  puts ("");
+	  binprint (strprep[i].out, strlen (strprep[i].out));
+	  puts ("");
+	}
+      else if (debug)
+	printf("returned %d expected %d\n", rc, strprep[i].rc);
+
       if (rc == STRINGPREP_OK)
 	{
 	  if (strlen (strprep[i].out) != strlen (p) ||
@@ -219,13 +250,21 @@ main (int argc, char *argv[])
 	      if (debug)
 		printf ("ERROR\n");
 	    }
+	  else if (debug)
+	    printf ("OK\n\n");
 	}
       else if (debug)
-	printf ("OK\n");
+	printf ("OK\n\n");
     }
 
   free (p);
 
+#if 0
+  memset(p, 0, 10);
+  stringprep_unichar_to_utf8 (0x0628, p);
+  hexprint (p, strlen (p)); puts("");
+#endif
+
   if (debug)
     printf ("Stringprep self tests done with %d errors\n", error_count);
 
